!= phy_lscond ⥸塼Υƥȥץ
!
!= Test program for "phy_lscond"
!
! Authors::   Yukiko YAMADA, Yasuhiro MORIKAWA
! Version::   $Id: phy_lscond_test.f90,v 1.8 2008-06-14 11:44:15 morikawa Exp $
! Tag Name::  $Name: dcpam4-20080626 $
! Copyright:: Copyright (C) GFD Dennou Club, 2007. All rights reserved.
! License::   See COPYRIGHT[link:../../../COPYRIGHT]
!
! <b>Note that Japanese and English are described in parallel.</b>
!
! phy_lscond ⥸塼ưƥȤԤΥץǤ.
! ΥץबѥǤ뤳, Ӽ¹Ի
! ץबｪλ뤳ȤǧƤ.
!
! This program checks the operation of "phy_lscond" module.
! Confirm compilation and execution of this program.
!

program phy_lscond_test
  use phy_lscond, only: PHYLSC, Create, Close, &
    & PutLine, initialized, LScaleCond
  use constants, only: CONST, Create, Get
  use dc_test, only: AssertEqual, AssertGreaterThan, AssertLessThan
  use dc_types, only: DP, STRING
  use dc_string, only: StoA, PutLine
  use dc_args, only: ARGS, Open, HelpMsg, Option, Debug, Help, Strict, Close
  use gt4_history, only: GT_HISTORY, &
    & HistoryCreate, HistoryAddVariable, HistoryPut, HistoryClose, &
    & HistoryAddAttr, HistoryGet
  implicit none

  !---------------------------------------------------------
  !  ¸ɽ, ǥ̾, °̾
  !  Title of a experiment, name of model, sub-organ
  !---------------------------------------------------------
  character(*), parameter:: title = &
    & 'phy_lscond_test $Name: dcpam4-20080626 $ :: ' // &
    & 'Test program of "phy_lscond" module'
  character(*), parameter:: source = &
    & 'dcpam4 ' // &
    & '(See http://www.gfd-dennou.org/library/dcpam)'
  character(*), parameter:: institution = &
    & 'GFD Dennou Club (See http://www.gfd-dennou.org)'

  !-------------------------------------------------------------------
  !  ʻȿ
  !  Grid points and maximum truncated wavenumber
  !-------------------------------------------------------------------
  integer, parameter:: imax = 32
                              ! ٳʻ. 
                              ! Number of grid points in longitude
  integer, parameter:: jmax = 16
                              ! ٳʻ. 
                              ! Number of grid points in latitude
  integer, parameter:: kmax = 8
                              ! ľؿ. 
                              ! Number of vertical level

  !-----------------------------------------------------------------
  !  ʪ
  !  Configure physical constants etc.
  !-----------------------------------------------------------------
  type(CONST):: const_earth
  real(DP):: Grav      ! $ g $ .      ϲ®.     Gravitational acceleration
  real(DP):: Cp        ! $ C_p $ .    絤갵Ǯ.   Specific heat of air at constant pressure
  real(DP):: EL        ! $ L $ .      ζŷǮ. Latent heat of condensation of water vapor
  real(DP):: RVap      ! $ R_v $ .    . Gas constant of water vapor
  real(DP):: EpsV      ! $ \epsilon_v $ .        ʬ. Molecular weight ratio of water vapor
  real(DP):: ES0       ! $ e^{*} $ (273K) .      0 Ǥ˰¾. Saturated vapor pressure at 0 degrees C
  real(DP):: DelTime    ! $ \Delta t $ . ॹƥå. Time step

  !-------------------------------------------------------------------
  !  ǡ
  !  Axes data
  !-------------------------------------------------------------------
  real(DP):: x_Lon (0:imax-1)
                              ! . Longitude
  real(DP):: y_Lat (0:jmax-1)
                              ! . Latitude
  real(DP):: z_Sigma (0:kmax-1)
                              ! $ \sigma $ ٥ (). 
                              ! Full $ \sigma $ level
  real(DP):: r_Sigma (0:kmax)
                              ! $ \sigma $ ٥ (Ⱦ). 
                              ! Half $ \sigma $ level
  real(DP):: x_Lon_Weight (0:imax-1)
                              ! ʬѺɸŤ. 
                              ! Weight for integration in longitude
  real(DP):: y_Lat_Weight (0:jmax-1)
                              ! ʬѺɸŤ. 
                              ! Weight for integration in latitude
  real(DP):: z_DelSigma (0:kmax-1)
                              ! $ \Delta \sigma $ (). 
                              ! $ \Delta \sigma $ (Full)

  !---------------------------------------------------------
  !  ʪ
  !  Physical values
  !---------------------------------------------------------
  real(DP), parameter:: PI = 3.1415926535897930_DP
                                ! $ \pi $ . ߼Ψ. Circular constant

  real(DP):: xy_Ps (0:imax-1, 0:jmax-1)
                              ! $ p_s $ .   ɽ̵. Surface pressure
  real(DP):: xyz_Temp (0:imax-1, 0:jmax-1, 0:kmax-1)
                              ! $ T $ .     . Temperature
  real(DP):: xyz_QVap (0:imax-1, 0:jmax-1, 0:kmax-1)
                              ! $ q $ .     漾. Specific humidity
  real(DP):: xyz_Press (0:imax-1, 0:jmax-1, 0:kmax-1)
                              ! $ p_s $ . ɽ̵ (٥). 
                              ! Surface pressure (full level)
  real(DP):: xyr_Press (0:imax-1, 0:jmax-1, 0:kmax)
                              ! $ p_s $ . ɽ̵ (Ⱦ٥). 
                              ! Surface pressure (half level)
  real(DP):: xy_LscRain (0:imax-1, 0:jmax-1)
                              ! 絬϶ŷ륹ˤ߿. 
                              ! Precipitation by large scale condensation scheme
  real(DP):: xyz_DLscTempDt (0:imax-1, 0:jmax-1, 0:kmax-1)
                              ! 絬϶ŷ륹ˤ벹ѲΨ. 
                              ! Temperature tendency by large scale condensation scheme
  real(DP):: xyz_DLscQVapDt (0:imax-1, 0:jmax-1, 0:kmax-1)
                              ! 絬϶ŷ륹ˤ漾ѲΨ. 
                              ! Specific humidity tendency by large scale condensation scheme


  real(DP):: xyz_TempAns (0:imax-1, 0:jmax-1, 0:kmax-1)
                              ! $ T $ .     . Temperature
  real(DP):: xyz_QVapAns (0:imax-1, 0:jmax-1, 0:kmax-1)
                              ! $ q $ .     漾. Specific humidity
  real(DP):: xyz_DLscTempDtAns (0:imax-1, 0:jmax-1, 0:kmax-1)
                              ! 絬϶ŷ륹ˤ벹ѲΨ. 
                              ! Temperature tendency by large scale condensation scheme
  real(DP):: xyz_DLscQVapDtAns (0:imax-1, 0:jmax-1, 0:kmax-1)
                              ! 絬϶ŷ륹ˤ漾ѲΨ. 
                              ! Specific humidity tendency by large scale condensation scheme
  real(DP):: xy_LscRainAns (0:imax-1, 0:jmax-1)
                              ! 絬϶ŷ륹ˤ߿. 
                              ! Precipitation by large scale condensation scheme

  !-------------------------------------------------------------------
  !  ǡ
  !  Data I/O
  !-------------------------------------------------------------------
  type(GT_HISTORY):: gthist

  !---------------------------------------------------------
  !  ѿ
  !  Work variables
  !---------------------------------------------------------
  integer:: i, j, k           ! DO 롼Ѻѿ
                              ! Work variables for DO loop

  type(ARGS):: arg            ! ޥɥ饤. 
                              ! Command line arguments
  logical:: OPT_namelist      ! -N, --namelist ץ̵ͭ. 
                              ! Existence of '-N', '--namelist' option
  character(STRING):: VAL_namelist
                              ! -N, --namelist ץ. 
                              ! Value of '-N', '--namelist' option
  type(PHYLSC):: phy_lsc00, phy_lsc01, phy_lsc02, phy_lsc03
  logical:: err
  character(*), parameter:: subname = 'phy_lscond_test'
continue

  !---------------------------------------------------------
  !  ޥɥ饤ν
  !  Command line arguments handling
  !---------------------------------------------------------
  call Open( arg )
  call HelpMsg( arg, 'Title', title )
  call HelpMsg( arg, 'Usage', &
    & './phy_lscond_test [Options]' )
  call HelpMsg( arg, 'Source', source )
  call HelpMsg( arg, 'Institution', institution )
  call Option( arg, StoA('-N', '--namelist'), &
    & OPT_namelist, VAL_namelist, help = 'NAMELIST filename' )
  call Debug( arg ) ; call Help( arg ) ; call Strict( arg, severe = .true. )
  call Close( arg )

  !---------------------------------------------------------
  !  ʪν
  !  Prepare physical constants
  !---------------------------------------------------------
  call Create( const_earth ) ! (inout)

  DelTime = 600.0_DP

  call Get( constant = const_earth, &      ! (inout)
    & Grav = Grav, Cp = Cp, &              ! (out)
    & EL = EL, RVap = RVap, &              ! (out)
    & EpsV = EpsV, ES0 = ES0 )             ! (out)

  !-------------------------------------------------------------------
  !  ǡ
  !  Configure axes data
  !-------------------------------------------------------------------
  call HistoryGet ( &
    & file = 'phy_lscond_test00.nc', varname = 'lon', & ! (in)
    & array = x_Lon, quiet = .true. )                 ! (out)
  x_Lon = x_Lon * PI / 180.0_DP

  call HistoryGet ( &
    & file = 'phy_lscond_test00.nc', varname = 'lon_weight', & ! (in)
    & array = x_Lon_Weight, quiet = .true. )                 ! (out)

  call HistoryGet ( &
    & file = 'phy_lscond_test00.nc', varname = 'lat', & ! (in)
    & array = y_Lat, quiet = .true. )                 ! (out)
  y_Lat = y_Lat * PI / 180.0_DP

  call HistoryGet ( &
    & file = 'phy_lscond_test00.nc', varname = 'lat_weight', & ! (in)
    & array = y_Lat_Weight, quiet = .true. )                 ! (out)

  call HistoryGet ( &
    & file = 'phy_lscond_test00.nc', varname = 'sig', & ! (in)
    & array = z_Sigma, quiet = .true. )                 ! (out)

  call HistoryGet ( &
    & file = 'phy_lscond_test00.nc', varname = 'sigm', & ! (in)
    & array = r_Sigma, quiet = .true. )                 ! (out)

  do k = 0, kmax - 1
    z_DelSigma(k) = r_Sigma(k) - r_Sigma(k+1)
  enddo

  !---------------------------------------------------------
  !  ƥ
  !  Initialization test
  !---------------------------------------------------------
  call Create( phy_lsc = phy_lsc00,  &         ! (inout)
    & imax = imax, jmax = jmax, kmax = kmax, & ! (in)
    & Grav = Grav, Cp = Cp, &                  ! (in)
    & EL = EL, RVap = RVap, &                  ! (in)
    & EpsV = EpsV, ES0 = ES0, &                ! (in)
    & DelTime = DelTime )
  call AssertEqual( 'initialization test 1', &
    & answer = .true., check = initialized(phy_lsc00) )
  call PutLine( phy_lsc = phy_lsc00 ) ! (in)

  !---------------------------------------------------------
  !  ǡ
  !  Input data
  !---------------------------------------------------------
  call HistoryGet ( &
    & file = 'phy_lscond_test00.nc', varname = 'Temp', & ! (in)
    & array = xyz_Temp, quiet = .true. )                ! (out)
  call HistoryGet ( &
    & file = 'phy_lscond_test00.nc', varname = 'Ps', & ! (in)
    & array = xy_Ps, quiet = .true. )                ! (out)
!!$  call HistoryGet ( &
!!$    & file = 'phy_lscond_test00.nc', varname = 'QVap', & ! (in)
!!$    & array = xyz_QVap, quiet = .true. )                ! (out)

  do k = 0, kmax - 1
    xyz_Press(:,:,k) = xy_Ps * z_Sigma(k)
  end do

  do k = 0, kmax
    xyr_Press(:,:,k) = xy_Ps * r_Sigma(k)
  end do

  xyz_QVap = 1.0e-2


  call HistoryGet ( &
    & file = 'phy_lscond_test01.nc', varname = 'Temp', & ! (in)
    & array = xyz_TempAns, quiet = .true. )                ! (out)
  call HistoryGet ( &
    & file = 'phy_lscond_test01.nc', varname = 'QVap', & ! (in)
    & array = xyz_QVapAns, quiet = .true. )                ! (out)
  call HistoryGet ( &
    & file = 'phy_lscond_test01.nc', varname = 'DLscTempDt', & ! (in)
    & array = xyz_DLscTempDtAns, quiet = .true. )                ! (out)
  call HistoryGet ( &
    & file = 'phy_lscond_test01.nc', varname = 'DLscQVapDt', & ! (in)
    & array = xyz_DLscQVapDtAns, quiet = .true. )                ! (out)
  call HistoryGet ( &
    & file = 'phy_lscond_test01.nc', varname = 'LscRain', & ! (in)
    & array = xy_LscRainAns, quiet = .true. )                ! (out)


  !---------------------------------------------------------
  !  LScaleCond ƥ
  !  LScaleCond test
  !---------------------------------------------------------

!!$  xyz_Temp = 273.0_DP
!!$  xyz_QVap = 1.0e-5_DP
!!$  xyz_Press = 1.0e+6_DP
!!$  xyr_Press = 1.0e+6_DP

  call LScaleCond( phy_lsc = phy_lsc00, &             ! (inout)
    & xyz_Temp = xyz_Temp, xyz_QVap = xyz_QVap, &     ! (inout)
    & xyz_Press = xyz_Press, xyr_Press = xyr_Press, & ! (in)
    & xy_Rain = xy_LscRain, &                         ! (out)
    & xyz_DTempDt = xyz_DLscTempDt, &                 ! (out)
    & xyz_DQVapDt = xyz_DLscQVapDt )                  ! (out)

!!$  call PutLine ( xyz_Temp, indent = ' xyz_Temp=')
!!$  call PutLine ( xyz_QVap, indent = ' xyz_QVap=')
!!$  call PutLine ( xy_LscRain, indent = ' xy_LscRain=')
!!$  call PutLine ( xyz_DLscTempDt, indent = ' xyz_DLscTempDt=')
!!$  call PutLine ( xyz_DLscQvapDt, indent = ' xyz_DLscQvapDt=')

  call AssertEqual( 'LScaleCond test (Temp) ', &
    & answer = xyz_TempAns, check = xyz_Temp, &
    & significant_digits = 14, ignore_digits = -15 )
  call AssertEqual( 'LScaleCond test (QVap) ', &
    & answer = xyz_QVapAns, check = xyz_QVap, &
    & significant_digits = 15, ignore_digits = -15 )
  call AssertEqual( 'LScaleCond test (DLscTempDt)', &
    & answer = xyz_DLscTempDtAns, check = xyz_DLscTempDt, &
    & significant_digits = 15, ignore_digits = -15 )
  call AssertEqual( 'LScaleCond test (DLscQVapDt)', &
    & answer = xyz_DLscQVapDtAns, check = xyz_DLscQVapDt, &
    & significant_digits = 15, ignore_digits = -15 )
  call AssertEqual( 'LScaleCond test (LscRain)', &
    & answer = xy_LscRainAns, check = xy_LscRain, &
    & significant_digits = 14, ignore_digits = -15 )

  !---------------------------------------------------------
  !  λƥ
  !  Termination test
  !---------------------------------------------------------
  call Close( phy_lsc = phy_lsc00 ) ! (inout)
  call AssertEqual( 'termination test 1', &
    & answer = .false., check = initialized(phy_lsc00) )
  call PutLine( phy_lsc = phy_lsc00 ) ! (in)

  call Close( phy_lsc = phy_lsc02, & ! (inout)
    & err = err )                            ! (out)
  call AssertEqual( 'termination test 2', &
    & answer = .true., check = err )


!!$  !----------------------------------------------------------------
!!$  !  ǡ
!!$  !  Output data
!!$  !----------------------------------------------------------------
!!$  call HistoryCreate( &
!!$    & history = gthist, &                                ! (out)
!!$    & file = 'phy_lscond_test01.nc', &        ! (in)
!!$    & title = title, &                                   ! (in)
!!$    & source = source, institution = institution, &      ! (in)
!!$    & dims = StoA('lon', 'lat', 'sig', 'sigm'), &        ! (in)
!!$    & dimsizes = (/imax, jmax, kmax, kmax + 1/), &       ! (in)
!!$    & longnames = &
!!$    &  StoA('longitude', 'latitude', &
!!$    &       'sigma at layer midpoints', &
!!$    &       'sigma at layer end-points (half level)'), & ! (in)
!!$    & units = StoA('degree_east', 'degree_north', &
!!$    &              '1', '1') )                           ! (out)
!!$
!!$  call HistoryPut( &
!!$    & history = gthist, &               ! (out)
!!$    & varname = 'lon', array = x_Lon * 180.0_DP / PI )  ! (in)
!!$
!!$  call HistoryPut( &
!!$    & history = gthist, &               ! (out)
!!$    & varname = 'lat', array = y_Lat * 180.0_DP / PI  ) ! (in)
!!$  call HistoryPut( &
!!$    & history = gthist, &                ! (out)
!!$    & varname = 'sig', array = z_Sigma ) ! (in)
!!$  call HistoryPut( &
!!$    & history = gthist, &                 ! (out)
!!$    & varname = 'sigm', array = r_Sigma ) ! (in)
!!$
!!$  call HistoryAddAttr( &
!!$    & history = gthist, &                    ! (inout)
!!$    & varname = 'lon', attrname = 'standard_name', & ! (in)
!!$    & value = 'longitude' )                          ! (in)
!!$  call HistoryAddAttr( &
!!$    & history = gthist, &                    ! (inout)
!!$    & varname = 'lat', attrname = 'standard_name', & ! (in)
!!$    & value = 'latitude' )                           ! (in)
!!$
!!$  call HistoryAddAttr( &
!!$    & history = gthist, &                              ! (inout)
!!$    & varname = 'sig', attrname = 'standard_name', &   ! (in)
!!$    & value = 'atmosphere_sigma_coordinate' )          ! (in)
!!$  call HistoryAddAttr( &
!!$    & history = gthist, &                              ! (inout)
!!$    & varname = 'sigm', attrname = 'standard_name', &  ! (in)
!!$    & value = 'atmosphere_sigma_coordinate' )          ! (in)
!!$  call HistoryAddAttr( &
!!$    & history = gthist, &                         ! (inout)
!!$    & varname = 'sig', attrname = 'positive', &   ! (in)
!!$    & value = 'down' )                            ! (in)
!!$  call HistoryAddAttr( &
!!$    & history = gthist, &                         ! (inout)
!!$    & varname = 'sigm', attrname = 'positive', &  ! (in)
!!$    & value = 'down' )                            ! (in)
!!$
!!$
!!$  call HistoryAddVariable( &
!!$    & history = gthist, &                  ! (inout)
!!$    & varname = 'Temp', &                  ! (in)
!!$    & dims = StoA('lon', 'lat', 'sig'), &  ! (in)
!!$    & longname = 'temperature', &          ! (in)
!!$    & units = 'K', xtype = 'double' )      ! (in)
!!$  call HistoryPut( &
!!$    & history = gthist, &                  ! (inout)
!!$    & varname = 'Temp', array = xyz_Temp ) ! (in)
!!$
!!$  call HistoryAddVariable( &
!!$    & history = gthist, &                  ! (inout)
!!$    & varname = 'QVap', &                  ! (in)
!!$    & dims = StoA('lon', 'lat', 'sig'), &  ! (in)
!!$    & longname = 'specific humidity', &          ! (in)
!!$    & units = 'kg kg-1', xtype = 'double' )      ! (in)
!!$  call HistoryPut( &
!!$    & history = gthist, &                  ! (inout)
!!$    & varname = 'QVap', array = xyz_QVap ) ! (in)
!!$
!!$  call HistoryAddVariable( &
!!$    & history = gthist, &                  ! (inout)
!!$    & varname = 'DLscTempDt', &            ! (in)
!!$    & dims = StoA('lon', 'lat', 'sig'), &  ! (in)
!!$    & longname = 'large-scale condensation heating', &          ! (in)
!!$    & units = 'K s-1', xtype = 'double' )      ! (in)
!!$  call HistoryPut( &
!!$    & history = gthist, &                  ! (inout)
!!$    & varname = 'DLscTempDt', array = xyz_DLscTempDt ) ! (in)
!!$
!!$  call HistoryAddVariable( &
!!$    & history = gthist, &                  ! (inout)
!!$    & varname = 'DLscQVapDt', &            ! (in)
!!$    & dims = StoA('lon', 'lat', 'sig'), &  ! (in)
!!$    & longname = 'large-scale condensation moistening', &  ! (in)
!!$    & units = 'kg kg-1 s-1', xtype = 'double' )      ! (in)
!!$  call HistoryPut( &
!!$    & history = gthist, &                  ! (inout)
!!$    & varname = 'DLscQVapDt', array = xyz_DLscQVapDt ) ! (in)
!!$
!!$  call HistoryAddVariable( &
!!$    & history = gthist, &                  ! (inout)
!!$    & varname = 'LscRain', &            ! (in)
!!$    & dims = StoA('lon', 'lat'), &  ! (in)
!!$    & longname = 'precipitation by large scale condensation', &  ! (in)
!!$    & units = 'W m-2', xtype = 'double' )      ! (in)
!!$  call HistoryPut( &
!!$    & history = gthist, &                  ! (inout)
!!$    & varname = 'LscRain', array = xy_LscRain ) ! (in)
!!$
!!$  call HistoryClose( history = gthist ) ! (inout)

end program phy_lscond_test
